.include "m8def.inc"
          .equ fCK         =  8000000             ;  
          .equ BAUD        =  9600                ;  UART  
          .equ UBRR_value  =  (fCK/(BAUD*16))-1   ;    UBRR

.CSEG

         .org 0         ; (RESET)
         rjmp start

         .org INT0addr
         rjmp INT0_vect ; (INT0) External Interrupt Request 0

         .org INT1addr
         rjmp INT1_vect ; (INT1) External Interrupt Request 1
         .ORG $006
         RETI           ; (TIMER2 COMP) Timer/Counter2 Compare Match
         .ORG $008
         RETI           ; (TIMER2 OVF) Timer/Counter2 Overflow
         .ORG $00A
         RETI           ; (TIMER1 CAPT) Timer/Counter1 Capture Event
         .ORG $00C
         RETI           ; (TIMER1 COMPA) Timer/Counter1 Compare Match A
         .ORG $00E
         RETI           ; (TIMER1 COMPB) Timer/Counter1 Compare Match B
         .ORG $010
         RETI           ; (TIMER1 OVF) Timer/Counter1 Overflow
         .ORG $012
         RETI           ; (TIMER0 OVF) Timer/Counter0 Overflow
         .ORG $014
         RETI           ; (SPI,STC) Serial Transfer Complete
         .ORG $016
         RJMP   RX_OK   ; (USART,RXC) USART, Rx Complete
         .ORG $018
         RETI           ; (USART,UDRE) USART Data Register Empty
         .ORG $01A
         RETI           ; (USART,TXC) USART, Tx Complete
         .ORG $01C
         RETI           ; (ADC) ADC Conversion Complete
         .ORG $01E
         RETI           ; (EE_RDY) EEPROM Ready
         .ORG $020
         RETI           ; (ANA_COMP) Analog Comparator
         .ORG $022
         RETI           ; (TWI) 2-wire Serial Interface
         .ORG $024
         RETI           ; (INT2) External Interrupt Request 2
         .ORG $026
         RETI           ; (TIMER0 COMP) Timer/Counter0 Compare Match
         .ORG $028
         RETI           ; (SPM_RDY) Store Program Memory Ready

start:
   ldi R16,LOW(RamEnd) ;  
   out SPL,R16
   ldi R16,HIGH(RamEnd)
   out SPH,R16

   sbi DDRC,0          ;  0 - 
   sbi DDRC,1          ;  1 - 
   sbi DDRC,2          ;  2 - 
   sbi DDRC,3          ;  3 - 
   sbi DDRC,4          ;  4 - 
   sbi DDRC,5          ;  5 - 

   cbi PORTC,0          ;  C0 = 0
   cbi PORTC,1          ;  C1 = 0
   cbi PORTC,2          ;  C2 = 0
   cbi PORTC,3          ;  C3 = 0
   cbi PORTC,4          ;  C4 = 0
   cbi PORTC,5          ;  C5 = 0

   sbi DDRB,0          ;  B0 - 
   sbi DDRB,1          ;  B1 - 
   sbi DDRB,2          ;  B2 - 
   sbi DDRB,3          ;  B3 - 
   sbi DDRB,4          ;  B4 - 
   sbi DDRB,5          ;  B5 - 
   sbi DDRB,6          ;  B6 - 
   sbi DDRB,7          ;  B7 - 

   sbi PORTB,0          ;  B0 - 
   sbi PORTB,1          ;  B1 - 
   sbi PORTB,2          ;  B2 - 
   sbi PORTB,3          ;  B3 - 
   sbi PORTB,4          ;  B4 - 
   sbi PORTB,5          ;  B5 - 
   sbi PORTB,6          ;  B6 - 
   sbi PORTB,7          ;  B7 - 

   cbi DDRD,2          ;  D3 - 
   sbi PORTD,2         ;  D3 -  
   cbi DDRD,3          ;  D3 - 
   sbi PORTD,3         ;  D3 -  


;  GICR:     76543210
;            1---------- INT0 
;             1--------- INT1 
;
;                  1---- IVSEL
;                   1--- IVCE
   ldi R17,0b11000000
   out GICR,R17

;  MCUCR:    76543210
;                XX ----  ISC11,ISC10     INT1
;                               0     0   Low level INT1
;                               0     1   Any logical change
;                               1     0   The falling edge
;                               1     1   The rising  edge
;                  XX --  ISC01,ISC00     INT0
;                               0     0   Low level INT1
;                               0     1   Any logical change
;                               1     0   The falling edge
;                               1     1   The rising  edge
   ldi R17,0b00000000
   out MCUCR,R17

   rcall init_USART
   sei ;   

      ldi   R16,'T'
      rcall USART_send
      ldi   R16,'E'
      rcall USART_send
      ldi   R16,'S'
      rcall USART_send
      ldi   R16,'T'
      rcall USART_send
      ldi   R16,':'
      rcall USART_send
      ldi   R16,' '
      rcall USART_send      ; SPACE

;==================================
main_loop:
   nop
   nop
   nop
   rjmp main_loop

;==================================
RX_OK:
   IN   R16,UDR    ; read UART buff
   OUT  PORTB, R16
   reti

;==================================
INT0_vect:
   CLI
   RCALL UART_Receive
   OUT   PORTB, R16
   RCALL USART_send
   sbic PORTC,0
   rjmp setC0_0
   rjmp setC0_1

setC0_0:
   cbi PORTC,0
   rjmp label0_end

setC0_1:
   sbi PORTC,0

label0_end:
   SEI
   ret

;==================================
INT1_vect:
   sbic PORTC,1
   rjmp setC1_0
   rjmp setC1_1

setC1_0:
   cbi PORTC,1
   rjmp label1_end

setC1_1:
   sbi PORTC,1

label1_end:
   reti

;  USART 
init_USART:
      ldi   R16,high(UBRR_value)   ;  9600 
      out   UBRRH,R16
      ldi   R16,low(UBRR_value)
      out   UBRRL,R16
                                    ; |(1<<RXCIE)
      ldi   R16,(1<<TXEN)|(1<<RXEN) ;    
      out   UCSRB,R16

      ldi   R16,(1<< URSEL)|(1<< UCSZ0)|(1<< UCSZ1)
      out   UCSRC,R16 ;  8  ,   ,  
      ret
;  
USART_send:
      sbis  UCSRA,UDRE            ;   UDRE  UCSRA   
      rjmp  USART_send
      out   UDR,R16               ;   UART,     UDR
      ret
;  
UART_Receive:
      SBIS  UCSRA, RXC
      RJMP  UART_Receive
      IN    R16, UDR
      ret
